//-------------------------------------------------------------------------------
//      CYWUSB6935M     WireleaaUSB TX RX TEST Program Ver1.01a
//      
//      
//                                              Software design    : Masaru Osuga
//                                              Last     UPDATE    :   2009/07/11 
//
// Reorganization history--------------------------------------------------------
// Ver1.00		2008/12/30		New Software release.
//    1.01      2009/07/09      RSSI Bug Fix. 
//                              SPIM Clock UP 1.5MHz -> 2MHz
//    1.01a     2009/07/11      Byte_READ()      Added wait.
//-------------------------------------------------------------------------------
//
#include <m8c.h>
#include "PSoCAPI.h"
#include "PSoCGPIOINT.h"
#include "cywusb6935def.h"

#define  TX_RX_MODE            0		// TX MODE = 0 or RX MODE 1 

#define	 CMD_SINGLE_READ	0x00
#define  CMD_SINGLE_WRITE	0x80
#define  CMD_BURST__READ	0x40
#define  CMD_BURST__WRITE	0xC0

#define		PD_Lo	Port_1_Data_SHADE &= ~PD_MASK;PRT1DR = Port_1_Data_SHADE
#define		PD_Hi	Port_1_Data_SHADE |=  PD_MASK;PRT1DR = Port_1_Data_SHADE
#define		SS_Lo	Port_1_Data_SHADE &= ~SS_MASK;PRT1DR = Port_1_Data_SHADE
#define		SS_Hi	Port_1_Data_SHADE |=  SS_MASK;PRT1DR = Port_1_Data_SHADE
#define	   RST_Lo	Port_1_Data_SHADE &=~RST_MASK;PRT1DR = Port_1_Data_SHADE
#define	   RST_Hi	Port_1_Data_SHADE |= RST_MASK;PRT1DR = Port_1_Data_SHADE

static BYTE status=0;
static RxData;
const  BYTE PnCodeTable[] =
{   0x6A, 0xE7, 0x01, 0xEA, 0x03, 0xFD, 0x13, 0xD2, // PN Code ID  0 - Index  0
    0xDC, 0xC0, 0x6B, 0xB8, 0x2B, 0x09, 0xBB, 0xB2, // PN Code ID  1 - Index  1
    0xA3, 0x1E, 0xF2, 0xA4, 0x31, 0x32, 0x7A, 0xB3, // PN Code ID  2 - Index  2
    0x44, 0x83, 0x3B, 0xDD, 0x14, 0xCF, 0x8E, 0xC9, // PN Code ID  3 - Index  3
    0x35, 0x35, 0x4E, 0xC5, 0xF3, 0x52, 0x47, 0xB0, // PN Code ID  4 - Index  4
    0x7C, 0x23, 0x8A, 0xCE, 0x45, 0x5C, 0x54, 0xD7, // PN Code ID  5 - Index  5
    0x81, 0xAC, 0xFB, 0x83, 0x7A, 0x9A, 0x61, 0xAC, // PN Code ID  6 - Index  6
    0x3C, 0x12, 0x5F, 0x9C, 0x39, 0x98, 0xF6, 0x8A, // PN Code ID  7 - Index  7
// Chip default
//	0x1E, 0x8B, 0x6A, 0x3D, 0xE0, 0xE9, 0xB2, 0x22  // PN Code ID  8 - Index  8
};

void wait(LONG l){while(l--);}
void SPIsend(BYTE bDataSend)
{
	while (!( SPIM_CONTROL_REG & SPIM_SPIM_TX_BUFFER_EMPTY ));
	SPIM_TX_BUFFER_REG= bDataSend;
    while (!( SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ));
	RxData = SPIM_RX_BUFFER_REG ;
}

BYTE Byte_READ(BYTE Add)
{	BYTE ResiveData;
	SS_Lo;
	SPIsend(CMD_SINGLE_READ|Add);
	SPIsend(0x00);
	wait(10);							// Added 2009/07/11
	ResiveData = RxData;
	SS_Hi;
	return ResiveData;
}
void Burst_READ(BYTE Add,BYTE *pData,BYTE Length)
{	BYTE loop;
	SS_Lo;
	SPIM_TX_BUFFER_REG = (CMD_BURST__READ|Add);
    while (!( SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ));
	for (loop = 0; loop < Length; ++loop){
		SPIM_TX_BUFFER_REG = 0xff;
		while (!( SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ));
		*pData++ = SPIM_RX_BUFFER_REG ;
	}
	SS_Hi;
}
void Byte_WRITE(BYTE Add,BYTE WriteData)
{	SS_Lo;
	SPIsend(CMD_SINGLE_WRITE|Add);
	SPIsend(WriteData);
	SS_Hi;
}
void Burst_WRITE(BYTE Add,BYTE *pData,BYTE Length)
{	BYTE loop;
	SS_Lo;
	pData += (Length-1);
	SPIM_TX_BUFFER_REG = (CMD_BURST__WRITE|Add);
    while (!( SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ));
	for (loop = 0; loop < Length; ++loop){
		SPIM_TX_BUFFER_REG = *pData--;
		while (!( SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ));
	}
	SS_Hi;
}
BYTE ID_READ(void)
{	BYTE ID_Data;
	ID_Data = Byte_READ(REG_ID);
	return ID_Data;
}
BYTE CH_READ(void)
{	BYTE CH_Data;
	CH_Data =Byte_READ(REG_CHANNEL);
	CH_Data &= mCHANNEL;
	return CH_Data;
}
void CH_WRITE(CH_Data)
{
	CH_Data &= mCHANNEL;
	Byte_WRITE(REG_CHANNEL,CH_Data);
}
void ReceiveON(void)
{	BYTE Data;
	Data = Byte_READ(REG_CONTROL);
	Byte_WRITE(REG_CONTROL,(Data |  bRX_ENABLE));
}
void ReceiveOFF(void)
{	BYTE Data;
	Data = Byte_READ(REG_CONTROL);
	Byte_WRITE(REG_CONTROL,(Data & ~bRX_ENABLE));
}
void TransmitON(void)
{	BYTE Data;
	Data = Byte_READ(REG_CONTROL);
	Byte_WRITE(REG_CONTROL,(Data |  bTX_ENABLE));
}
void TransmitOFF(void)
{	BYTE Data;
	Data = Byte_READ(REG_CONTROL);
	Byte_WRITE(REG_CONTROL,(Data & ~bTX_ENABLE));
}
/*BYTE RSSI_READ(void)
{	BOOL Valid;
	BYTE RSSI_Data;
	RSSI_Data = Byte_READ(REG_RSSI);
	Valid = (RSSI_Data & bRSSI_VALID) ? TRUE : FALSE;
	RSSI_Data &= mRSSI;
	//return Valid; //if BOOL return to use. 
	return RSSI_Data;
}*/
BYTE RSSI_READ(void)        //Bug Fix 2009/07/09
{	BYTE RSSI_Data;
	ReceiveOFF();
	ReceiveON();
	RSSI_Data = Byte_READ(REG_RSSI) & mRSSI;
	return RSSI_Data;
}
void PN_CODE_READ(void)
{	BYTE loop,Length=8;
	BYTE PN_Code[8],PN_CodeD[8];
	Burst_READ(REG_PN_CODE,PN_CodeD,8);
	for (loop = 0; loop < Length; ++loop){
		PN_Code[loop] = PN_CodeD[7-loop];
		LCD_Position(1,loop*2);LCD_PrHexByte(PN_Code[loop]);
	}
}
void PN_CODE_SET(BYTE  PN_CodeID)
{	BYTE loop,Length=8;
	BYTE PN_Code[8];
	for (loop = 0; loop < Length; ++loop){
		PN_Code[loop] = PnCodeTable[PN_CodeID * 8 + loop];
	}
	Burst_WRITE(REG_PN_CODE,PN_Code,8);
}
BOOL RX_Read_Chack(void)
{	BOOL Valid;
	BYTE Status;
	Status = Byte_READ(REG_RX_INT_STAT);
	Valid  = ( Status & bFULL_A ) ? TRUE : FALSE;
	return Valid;
}

BYTE RX_Data_Read(void)
{	BYTE RX_Data,count;
	while(!(Byte_READ(REG_RX_INT_STAT) & bFULL_A)){
		count++;wait(1000);
		if(count>100){
			LCD_Position(1,0);LCD_PrCString("RX: **TIME UP** ");
			return 0;
		}
	}
	RX_Data = Byte_READ(REG_RX_DATA_A);
	return RX_Data;
}
void TX_Data_Write(BYTE TX_Data)
{	BYTE Valid,Status;
	while(!(Byte_READ(REG_TX_INT_STAT) & bTX_EMPTY));//wait for TX data register
	Status = Byte_READ(REG_TX_INT_STAT);
	Byte_WRITE(REG_TX_DATA,TX_Data);
	Byte_WRITE(REG_TX_VALID,0xFF);
	Valid = Byte_READ(REG_TX_VALID);
}
void TX_PA_SET(BYTE PA)
{	BYTE Data;
	Data = Byte_READ(REG_CONTROL);
	Byte_WRITE(REG_CONTROL,(Data |  bPA_ENABLE | bAUTO_PA_DISABLE));
	Byte_WRITE(REG_PA,PA);
}
BYTE TX_PA_READ(void)
{	BYTE PA;
	PA = Byte_READ(REG_PA);
	return PA;
}
void SERDES_Enable(void)
{	BYTE Data;
	Data = Byte_READ(REG_SERDES_CTL) & mEND_OF_FRAME_LEN;
	Byte_WRITE(REG_SERDES_CTL,Data | bSERDES_ENABLE);
}
void RadioCLOCK_Init(void)
{
	Byte_WRITE(REG_CLOCK_MANUAL,CM_DEFAULT);
	Byte_WRITE(REG_CLOCK_ENABLE,CE_DEFAULT);
}

void main()
{
	BYTE Channel =     5;
	BYTE TX_Power=     0;
	BYTE TX_D    =     0;
	int  count   = 0x100;
	M8C_EnableGInt;
    LCD_Start();
	SPIM_Start(SPIM_SPIM_MODE_0 | SPIM_SPIM_MSB_FIRST);
	PD_Hi;
	RST_Hi;RST_Lo;wait(500);RST_Hi;
	RadioCLOCK_Init();
	
  #if	TX_RX_MODE
	ReceiveON();TransmitOFF();	// RX TEST
  #else
	ReceiveOFF();TransmitON();	// TX TEST
	TX_PA_SET(TX_Power);      
  #endif
  
	SERDES_Enable();
	ID_READ();
	CH_WRITE(Channel);
	CH_READ();
	PN_CODE_SET(2);
	while(1){
   
   #if	TX_RX_MODE				// RX TEST
		LCD_Position(0,0);
		LCD_PrCString("CH:");	LCD_PrHexByte(CH_READ());LCD_PrCString("   ");
		LCD_PrCString("RSSI:");	LCD_PrHexByte(RSSI_READ());
		LCD_Position(1,0);
		LCD_PrCString("RX:");	LCD_PrHexByte(RX_Data_Read());
		LCD_PrCString(" v:");	LCD_PrHexByte(Byte_READ(REG_RX_VALID_A));
		LCD_PrCString(" s:");	LCD_PrHexByte(Byte_READ(REG_RX_INT_STAT));
		
   #else						// TX TEST
		LCD_Position(0,0);
		LCD_PrCString("CH:");	LCD_PrHexByte(CH_READ());LCD_PrCString("    ");
		LCD_PrCString("PA:");	LCD_PrHexByte(TX_PA_READ());
		TX_Data_Write(TX_D);
		LCD_Position(1,0);
		LCD_PrCString("TX:");	LCD_PrHexByte(TX_D++);
		LCD_PrCString(" v:");	LCD_PrHexByte(Byte_READ(REG_TX_VALID));
		LCD_PrCString(" s:");	LCD_PrHexByte(Byte_READ(REG_TX_INT_STAT));
		wait(3000L);
   #endif
		
	}
}
